unit GPIB;
// Powered by Jan H. Kuypers
// Berkeley Micromechanical Analysis and Design Group (BMAD)
// University of California at Berkeley, April 2008
// 07-26-12 Daniel Kruger added ansichar and ansisting type definitions

interface

uses
   Windows, SysUtils, Dialogs, Math;

const
  (* GPIB status bit definitions.                                        *)
  ERR     = $8000;           (* Error detected                           *)
  TIMO    = $4000;           (* Timeout                                  *)
  ENDgpib = $2000;           (* EOI or EOS detected                      *)
  SRQI    = $1000;           (* SRQ detected by CIC                      *)
  RQS     =  $800;           (* Device needs service                     *)
  SPOLL   =  $400;           (* Board has been serially polled           *)
  EVENT   =  $200;           (* An event has occurred                    *)
  CMPL    =  $100;           (* I/O completed                            *)
  LOK     =   $80;           (* Local lockout state                      *)
  REM     =   $40;           (* Remote state                             *)
  CIC     =   $20;           (* Controller-in-charge                     *)
  ATN     =   $10;           (* Attention asserted                       *)
  TACS    =    $8;           (* Talker active                            *)
  LACS    =    $4;           (* Listener active                          *)
  DTAS    =    $2;           (* Device trigger state                     *)
  DCAS    =    $1;           (* Device clear state                       *)

  (* Error messages returned in global variable IBERR:                   *)
  EDVR =  0;                 (* System error                             *)
  ECIC =  1;                 (* Function requires GPIB board to be CIC   *)
  ENOL =  2;                 (* Write function detected no Listeners     *)
  EADR =  3;                 (* Interface board not addressed correctly  *)
  EARG =  4;                 (* Invalid argument to function call        *)
  ESAC =  5;                 (* Function requires GPIB board to be SAC   *)
  EABO =  6;                 (* I/O operation aborted                    *)
  ENEB =  7;                 (* Non-existent interface board             *)
  EDMA =  8;                 (* Error performing DMA                     *)
  EOIP = 10;                 (* I/O operation started before previous    *)
			     (* operation completed                      *)
  ECAP = 11;                 (* No capability for intended operation     *)
  EFSO = 12;                 (* File system operation error              *)
  EBUS = 14;                 (* Command error during device call         *)
  ESTB = 15;                 (* Serial poll status byte lost             *)
  ESRQ = 16;                 (* SRQ remains asserted                     *)
  ETAB = 20;                 (* The return buffer is full                *)

  (* Values for the 488.2 Send command.                                  *)
  NULLend = $00;             (* Do nothing at the end of a transfer.     *)
  NLend   = $01;             (* Send NL with EOI after a transfer.       *)
  DABend  = $02;             (* Send EOI with the last DAB.              *)
  STOPend = $100;            (* Value used by the 488.2 Receive command. *)

  MAVbit  = $10;             (* Position of the Message Available bit.   *)

  (* Timeout values and meanings:                                        *)
  TNONE   =  0;              (* Infinite timeout (disabled)              *)
  T10us   =  1;              (* Timeout of 10 microseconds (ideal)       *)
  T30us   =  2;              (* Timeout of 30 microseconds (ideal)       *)
  T100us  =  3;              (* Timeout of 100 microseconds (ideal)      *)
  T300us  =  4;              (* Timeout of 300 microseconds (ideal)      *)
  T1ms    =  5;              (* Timeout of 1 milliseconds (ideal)        *)
  T3ms    =  6;              (* Timeout of 3 milliseconds (ideal)        *)
  T10ms   =  7;              (* Timeout of 10 milliseconds (ideal)       *)
  T30ms   =  8;              (* Timeout of 30 milliseconds (ideal)       *)
  T100ms  =  9;              (* Timeout of 100 milliseconds (ideal)      *)
  T300ms  = 10;              (* Timeout of 300 milliseconds (ideal)      *)
  T1s     = 11;              (* Timeout of 1 second (ideal)              *)
  T3s     = 12;              (* Timeout of 3 seconds (ideal)             *)
  T10s    = 13;              (* Timeout of 10 seconds (ideal)            *)
  T30s    = 14;              (* Timeout of 30 seconds (ideal)            *)
  T100s   = 15;              (* Timeout of 100 seconds (ideal)           *)
  T300s   = 16;              (* Timeout of 300 seconds (ideal)           *)
  T1000s  = 17;              (* Timeout of 1000 seconds (ideal)          *)

type
  AddrList = array [0..31] of SmallInt;

  (* Type declarations for exported NI-488.2 Global Variables.             *)
  Tibsta  = function : integer; stdcall;
  Tiberr  = function : integer; stdcall;
  Tibcntl = function : Longint; stdcall;

  (* Type declarations for exported NI-488.2 routines.                     *)
   TDevClear = procedure (boardID: integer;
		                    address: word); stdcall;

   TFindLstn = procedure (boardID:    integer;
                          padlist:    AddrList;
                          resultlist: AddrList;
                          limit:      integer); stdcall;

   Tibonl = function (ud: integer;
                      v:  integer) : integer; stdcall;

   TReadStatusByte = procedure (boardID:    integer;
                                address:    word;
	                        var result: integer); stdcall;

   TReceive = procedure (boardID:     integer;
	                 address:     word;
	                 var buffer;
	                 cnt:         longint;
	                 termination: integer); stdcall;

   TSend = procedure (boardID: integer;
	              address: word;
	              var buffer;
	              datacnt: longint;
	              eotmode: integer); stdcall;

   TSendIFC = procedure (boardID: integer); stdcall;

   TWaitSRQ = procedure (boardID:    integer;
                         var result: integer); stdcall;
   GPIBchars = packed array[0..100] of ansiChar; // GPIB input output data
   Tibfind = function (BoardID: ansistring) : integer; stdcall;

var
  // used for GPIB
  instruments : AddrList;
  results : AddrList;

(* NI-488.2 GPIB global status variables                                 *)
  AddrIbsta  : Tibsta;
  AddrIberr  : Tiberr;
  AddrIbcntl : Tibcntl;

(* Pointers to the NI-488.2 GPIB global status variables                 *)
  Pibsta : ^integer;
  Piberr : ^integer;
  Pibcntl : ^Longint;

(* Declaration for the Handle for the GPIB library                       *)                 
  Gpib32Lib: THandle;

(* Declarations for the NI-488.2 GPIB calls                              *)
  DevClear: TDevClear;
  FindLstn: TFindLstn;
  ReadStatusByte: TReadStatusByte;
  Receive: TReceive;
  Send: TSend;
  SendIFC: TSendIFC;
  WaitSRQ: TWaitSRQ;

(* Declaration for the NI-488 GPIB call                                  *)
  ibonl: Tibonl;
  ibfind: Tibfind;

procedure StartUpGPIB;
procedure StopGPIB;
procedure SendGPIB(boardID: Integer; GPIBno : SmallInt; buf: GPIBchars);
function ReadGPIBReal(boardID: Integer; GPIBno : SmallInt; buf: GPIBchars):Real;
function FindBoard(boardID: Integer): Integer;
function ReadGPIBString(boardID: Integer; GPIBno : SmallInt; buf: GPIBchars):String;
function ListGPIBDevices(boardID: Integer): AddrList;

implementation
(* =====================================================================
 *                      Procedure loadDLL
 * This procedure loads the GPIB-32.DLL library. If the LoadLibrary
 * call is successful, the next step is to get the addresses of the
 * global status variables and functions using GetProcAddress. If the
 * GetProcAddress calls were successful, the procedure returns to the
 * main routine. Otherwise, it will free the library and HALT.
 *
 * The HALT function will terminate this program.
 * ===================================================================== *)
procedure loadDLL;
var
  str : string;

  begin
  (* Load the GPIB-32.DLL library using the LoadLibrary function.        *)
    Gpib32Lib := LoadLibrary('GPIB-32.DLL');

  (* Check to see if library loaded successfully. If the library could   *)
  (* not be loaded, display an error message and then HALT the program.  *)
    If Gpib32Lib = 0 Then
      Begin
        str := 'LoadLibrary FAILED!';
        MessageDlg(str, mtError, [mbOK], 0);
        halt;
      End;
  (* Get the addresses of the GPIB Global Variables.                     *)
    @AddrIbsta  := GetProcAddress(Gpib32Lib, 'user_ibsta');
    @AddrIberr  := GetProcAddress(Gpib32Lib, 'user_iberr');
    @AddrIbcntl := GetProcAddress(Gpib32Lib, 'user_ibcnt');

  (* Get the addresses of the functions needed for this application.     *)
    @DevClear       := GetProcAddress(Gpib32Lib, 'DevClear');
    @FindLstn       := GetProcAddress(Gpib32Lib, 'FindLstn');
    @ibonl          := GetProcAddress(Gpib32Lib, 'ibonl');
    @ReadStatusByte := GetProcAddress(Gpib32Lib, 'ReadStatusByte');
    @Receive        := GetProcAddress(Gpib32Lib, 'Receive');
    @Send           := GetProcAddress(Gpib32Lib, 'Send');
    @SendIFC        := GetProcAddress(Gpib32Lib, 'SendIFC');
    @WaitSRQ        := GetProcAddress(Gpib32Lib, 'WaitSRQ');
    @ibfind         := GetProcAddress(Gpib32Lib, 'ibfind');

  (* Verify that addresses were obtained. If unable to get any one of    *)
  (* the addresses, then free the library, display an error message and  *)
  (* HALT the program.                                                   *)
    if (@AddrIbsta      = NIL) Or
       (@AddrIberr      = NIL) Or
       (@AddrIbcntl     = NIL) Or
       (@DevClear       = NIL) Or
       (@FindLstn       = NIL) Or
       (@ibonl          = NIL) Or
       (@ReadStatusByte = NIL) Or
       (@Receive        = NIL) Or
       (@Send           = NIL) Or
       (@SendIFC        = NIL) Or
       (@WaitSRQ        = NIL) Then
       Begin
         str := 'GetProcAddresses FAILED!';
         MessageDlg(str, mtError, [mbOK], 0);
         (* Free the GPIB library.                                       *)
         FreeLibrary(Gpib32Lib);
         halt;
       End;

  (* Initialize GPIB global pointers to point to address location        *)
       Pibsta := @AddrIbsta;
       Piberr := @AddrIberr;
       Pibcntl := @AddrIbcntl;
  end;

(* =====================================================================
 *                       Procedure GPIBERR
 * This procedure will notify you that a NI-488.2 function failed by
 * printing an error message. The status variable IBSTA will be printed
 * in hexadecimal along with the mnemonic meaning of the bit position. The
 * status variable IBERR will be printed in decimal along with the
 * mnemonic meaning. The status variable will be printed in decimal.
 *
 * The NI-488 function ibonl is called to disable the hardware and
 * software.
 *
 * The HALT command stops execution of this program.
 * ===================================================================== *)
procedure gpiberr(msg: string);
  var
    str       : string;      (* String used for displaying messages.     *)
    ibstaStr  : string;      (* String for IBSTA.                        *)
    iberrStr  : string;      (* String for IBERR.                        *)
    ibcntlStr : string;      (* String for IBCNTL.                       *)

  begin
    ibstaStr := IntToHex(Pibsta^, 4);
    iberrStr := IntToStr(Piberr^);
    str := msg + '  ibsta = $' + ibstaStr + '  iberr = ' + iberrStr;
    MessageDlg(str, mtError, [mbOK], 0);

    str := 'ibsta ===> ';
    if (Pibsta^ and ERR) <> 0 Then
      str := Concat(str, '  ERR');
    if (Pibsta^ and TIMO) <> 0 Then
      str := Concat(str, '  TMO');
    if (Pibsta^ and ENDgpib) <> 0 Then
      str := Concat(str, '  END');
    if (Pibsta^ and SRQI) <> 0 Then
      str := Concat(str, '  SRQI');
    if (Pibsta^ and RQS) <> 0 Then
      str := Concat(str, '  RQS');
    if (Pibsta^ and SPOLL) <> 0 Then
      str := Concat(str, '  SPOLL');
    if (Pibsta^ and EVENT) <> 0 Then
      str := Concat(str, '  EVENT');
    if (Pibsta^ and CMPL) <> 0 Then
      str := Concat(str, '  CMPL');
    if (Pibsta^ and LOK) <> 0 Then
      str := Concat(str, '  LOK');
    if (Pibsta^ and REM) <> 0 Then
      str := Concat(str, '  REM');
    if (Pibsta^ and CIC) <> 0 Then
      str := Concat(str, '  CIC');
    if (Pibsta^ and ATN) <> 0 Then
      str := Concat(str, '  ATN');
    if (Pibsta^ and TACS) <> 0 Then
      str := Concat(str, '  TACS');
    if (Pibsta^ and LACS) <> 0 Then
      str := Concat(str, '  LACS');
    if (Pibsta^ and DTAS) <> 0 Then
      str := Concat(str, '  DTAS');
    if (Pibsta^ and DCAS) <> 0 Then
      str := Concat(str, '  DCAS');

    MessageDlg(str, mtError, [mbOK], 0);

    str := 'iberr ===> ';
    if Piberr^ = EDVR Then
      str := Concat(str, '  EDVR');
    if Piberr^ = ECIC Then
      str := Concat(str, '  ECIC');
    if Piberr^ = ENOL Then
      str := Concat(str, '  ENOL');
    if Piberr^ = EADR Then
      str := Concat(str, '  EADR');
    if Piberr^ = EARG Then
      str := Concat(str, '  EARG');
    if Piberr^ = ESAC Then
      str := Concat(str, '  ESAC');
    if Piberr^ = EABO Then
      str := Concat(str, '  EABO');
    if Piberr^ = ENEB Then
      str := Concat(str, '  ENEB');
    if Piberr^ = EDMA Then
      str := Concat(str, '  EDMA');
    if Piberr^ = EOIP Then
      str := Concat(str, '  EOIP');
    if Piberr^ = ECAP Then
      str := Concat(str, '  ECAP');
    if Piberr^ = EFSO Then
      str := Concat(str, '  EFSO');
    if Piberr^ = EBUS Then
      str := Concat(str, '  EBUS');
    if Piberr^ = ESTB Then
      str := Concat(str, '  ESTB');
    if Piberr^ = ESRQ Then
      str := Concat(str, '  ESRQ');
    if Piberr^ = ETAB Then
      str := Concat(str, '  ETAB');

    MessageDlg(str, mtError, [mbOK], 0);

    ibcntlStr := IntToStr(Pibcntl^);
    str :=  'ibcntl = ' + ibcntlStr;
    MessageDlg(str, mtError, [mbOK], 0);

  (* Call the ibonl function to disable the hardware and software.       *)
    ibonl(0, 0);
  (* Initialize GPIB global pointers to point to address location        *)
    FreeLibrary(Gpib32Lib);

    halt;
end;

procedure StartUpGPIB;
begin
   //********************
   //start up GPIB
   //********************

   // start up sequence
   (* Load the GPIB-32.DLL Library.                                         *)
   loadDLL;

   (* Your board needs to be the Controller-In-Charge in order to find    *)
   (* all listeners on the GPIB. To accomplish this, the function         *)
   (* SendIFC is called.                                                  *)
   SendIFC(0);
   if (Pibsta^ AND ERR) <> 0 THEN
   begin
    ShowMessage('No GPIB devices found.');
   end;
end;

procedure StopGPIB;
begin
   // close GPIB driver
   // Exit driver etc.
   (* Call the ibonl function to disable the hardware and software.    *)
   ibonl(0, 0);
   (* Free the GPIB library.                                           *)
   FreeLibrary(Gpib32Lib);
end;

procedure SendGPIB(boardID: Integer; GPIBno : SmallInt; buf: GPIBchars);
// Not used in the Demo, used for GPIB equipment control
const
  RxBufferSize = 256;
  TxBufferSize = 256;
begin
   (*clear device*)
   DevClear(boardID, GPIBno);
   if (Pibsta^ AND ERR) <> 0 then gpiberr('DevClear Error');
   Send(boardID, GPIBno, buf, 30, NLend);
   if (Pibsta^ AND ERR) <> 0 then gpiberr('Send Error');
end;

function ReadGPIBReal(boardID: Integer; GPIBno : SmallInt; buf: GPIBchars):Real;
// Not used in the Demo.
// Used for reading real data formated in engineering format from GPIB equipment
var
   j : integer;
   resultR,exp : Real;
   resultS : String;
begin
   (*clear device*)
   DevClear(boardID, GPIBno);
   // Send command
   Send(boardID, GPIBno, buf, 30, NLend);
   if (Pibsta^ AND ERR) <> 0 then gpiberr('Send Error');
   Receive(boardID, GPIBno, buf, 20, STOPend);
   (*take buf an convert to float and reformat to realistic resistance*)
   (*find position of E*)
   resultS := '';
   for j := 0 to 20 do
   begin
    If buf[j]='E' then Break
   else
    resultS := resultS+buf[j];
   end;
    // get exponent, last 2~3 digits after E
    exp := StrToFloat(buf[j+1]+buf[j+2]+buf[j+3]);
    //apply exponent
    resultR := Power(10,exp)*StrToFloat(resultS);
    Result := resultR;
end;

function FindBoard(boardID: Integer): Integer;
// Used to determine wether a GPIB BoardID is present or not
var
   GPIBBoardID : integer;
begin
   GPIBBoardID := ibfind('GPIB'+IntToStr(boardID));
   Result := GPIBBoardID;
end;

function ReadGPIBString(boardID: Integer; GPIBno : SmallInt; buf: GPIBchars):String;
// Not used in the Demo.
// Used for reading a string from GPIB equipment
var
   j : integer;
   resultR,exp : Real;
   resultS : String;
begin
   // Clear device
   DevClear(boardID, GPIBno);
   // Send command
   Send(boardID, GPIBno, buf, 100, NLend);
   if (Pibsta^ AND ERR) <> 0 then gpiberr('Send Error');
   Receive(boardID, GPIBno, buf, 100, STOPend);
   Result := buf;
end;

function ListGPIBDevices(boardID: Integer): AddrList;
// Checks for equipment present on a given GPIB BoardID
var
   i : integer;
   DeviceList: AddrList;
   AdressList: AddrList;
begin
  for i := 1 to 32 do
  begin
    AdressList[i-1] := i;
    DeviceList[i-1] := 0;
  end;
   // List of all possible devices
   FindLstn(boardID,AdressList,DeviceList,32);
   Result := DeviceList;
end;

end.
